#include <gtest/gtest.h>
#include "Vector3.h"

TEST(Vector3, Magnitude) {
	Vector3 v = Vector3(1, 2, 3);
	float m = 0;

	m = v.magnitude();
	EXPECT_FLOAT_EQ(m, 3.74165738677F) << "v.magnitude 1 2 3";

	m = Vector3::Magnitude(v);
	EXPECT_FLOAT_EQ(m, 3.74165738677F) << "Vector3::Magnitude 1 2 3";

	v = Vector3(-1, -2, -3);
	m = v.magnitude();
	EXPECT_FLOAT_EQ(m, 3.74165738677F) << "v.magnitude -1 -2 -3";

	v = Vector3(0, 0, 0);
	m = v.magnitude();
	EXPECT_FLOAT_EQ(m, 0) << "v.magnitude 0 0 0 ";

	v = Vector3(INFINITY, INFINITY, INFINITY);
	m = v.magnitude();
	EXPECT_FLOAT_EQ(m, INFINITY) << "v.magnitude INFINITY INFINITY INFINITY ";
}

TEST(Vector3, SqrMagnitude) {
	Vector3 v = Vector3(1, 2, 3);
	float m = 0;

	m = v.sqrMagnitude();
	EXPECT_FLOAT_EQ(m, 14) << "v.sqrMagnitude 1 2 3";

	m = Vector3::SqrMagnitude(v);
	EXPECT_FLOAT_EQ(m, 14) << "Vector3::SqrMagnitude 1 2 3";

	v = Vector3(-1, -2, -3);
	m = v.sqrMagnitude();
	EXPECT_FLOAT_EQ(m, 14) << "v.sqrMagnitude -1 -2 -3";

	v = Vector3(0, 0, 0);
	m = v.sqrMagnitude();
	EXPECT_FLOAT_EQ(m, 0) << "v.sqrMagnitude 0 0 0 ";
}

TEST(Vector3, Normalize) {
	bool r = false;

	Vector3 v = Vector3(0, 2, 0);
	Vector3 normalized_v = Vector3::zero;

	normalized_v = v.normalized();
	EXPECT_TRUE(normalized_v == Vector3(0, 1, 0)) << "v.normalized 0 2 0";

	normalized_v = Vector3::Normalize(v);
	EXPECT_TRUE(normalized_v == Vector3(0, 1, 0)) << "Vector3::Normalize 0 2 0";

	v = Vector3(0, -2, 0);
	normalized_v = v.normalized();
	EXPECT_TRUE(normalized_v == Vector3(0, -1, 0)) << "v.normalized 0 -2 0";
}

TEST(Vector3, Negate) {
	bool r = false;

	Vector3 v1 = Vector3(4, 5, 6);
	Vector3 v = Vector3::zero;

	v = -v1;
	EXPECT_TRUE(v == Vector3(-4, -5, -6)) << "- 4 5 6";

	v1 = Vector3(0, 0, 0);
	v = -v1;
	EXPECT_TRUE(v == Vector3(0, 0, 0)) << "- 0 0 0 ";
}

TEST(Vector3, Subtract) {
	bool r = false;

	Vector3 v1 = Vector3(4, 5, 6);
	Vector3 v2 = Vector3(1, 2, 3);
	Vector3 v = Vector3::zero;

	v = v1 - v2;
	EXPECT_TRUE(v == Vector3(3, 3, 3)) << "4 5 6 - 1 2 3";

	v2 = Vector3(-1, -2, -3);
	v = v1 - v2;
	EXPECT_TRUE(v == Vector3(5, 7, 9)) << "4 5 6 - -1 -2 -3";

	v2 = Vector3(4, 5, 6);
	v = v1 - v2;
	EXPECT_TRUE(v == Vector3(0, 0, 0)) << "4 5 6 - 4 5 6";
}

TEST(Vector3, Addition) {
	bool r = false;

	Vector3 v1 = Vector3(4, 5, 6);
	Vector3 v2 = Vector3(1, 2, 3);
	Vector3 v = Vector3::zero;

	v = v1 + v2;
	EXPECT_TRUE(v == Vector3(5, 7, 9)) << "4 5 6 + 1 2 3";

	v2 = Vector3(-1, -2, -3);
	v = v1 + v2;
	EXPECT_TRUE(v == Vector3(3, 3, 3)) << "4 5 6 + -1 -2 -3";

	v2 = Vector3(4, 5, 6);
	v = v1 + v2;
	EXPECT_TRUE(v == Vector3(8, 10, 12)) << "4 5 6 + 4 5 6";
}

TEST(Vector3, Scale) {
	bool r = false;

	Vector3 v1 = Vector3(4, 5, 6);
	Vector3 v2 = Vector3(1, 2, 3);
	Vector3 v = Vector3::zero;

	v = Vector3::Scale(v1, v2);
	EXPECT_TRUE(v == Vector3(4, 10, 18)) << "Scale 4 5 6 , 1 2 3";

	v2 = Vector3(-1, -2, -3);
	v = Vector3::Scale(v1, v2);
	EXPECT_TRUE(v == Vector3(-4, -10, -18)) << "Scale 4 5 6 , -1 -2 -3";

	v2 = Vector3(4, 5, 6);
	v = Vector3::Scale(v1, v2);
	EXPECT_TRUE(v == Vector3(16, 25, 36)) << "Scale 4 5 6 , 4 5 6";
}


TEST(Vector3, Multiply) {
	bool r = false;

	Vector3 v1 = Vector3(4, 5, 6);
	float f = 3;
	Vector3 v = Vector3::zero;

	v = v1 * f;
	EXPECT_TRUE(v == Vector3(12, 15, 18)) << "4 5 6 * 3";

	f = -3;
	v = v1 * f;
	EXPECT_TRUE(v == Vector3(-12, -15, -18)) << "4 5 6 * -3";

	f = 0;
	v = v1 * f;
	EXPECT_TRUE(v == Vector3(0, 0, 0)) << "4 5 6 * 0";

	f = INFINITY;
	v = v1 * f;
	EXPECT_TRUE(v == Vector3(INFINITY, INFINITY, INFINITY)) << "4 5 6 * INFINITY";

	f = -INFINITY;
	v = v1 * f;
	EXPECT_TRUE(v == Vector3(-INFINITY, -INFINITY, -INFINITY)) << "4 5 6 * -INFINITY";
}
